package it.unibas.utilita;

/**
 * Questa classe consente di effettuare operazioni di logging
 * su console e su file utilizzando la libreria java.util.logging
 * Sono previsti quattro diversi livelli di logging 
 * (in ordine di rilevanza decrescente):
 * <ul>
 * <li>SEVERE</li>
 * <li>INFO (livello standard)</li>
 * <li>FINE</li>
 * <li>FINER</li>
 * </ul>
 * E' possibile inoltre specificare i livelli off e all
 * Il livello di default (stabilito in %JRE_HOME%\lib\logging.properties) e' INFO
 * Per stabilire un livello di logging diverso utilizzare l'opzione: 
 * -Dit.unibas.utilita.Debug=<livello>
 * es: java -Dit.unibas.utilita.Debug=fine
 * E' possibile scegliere se effettuare il logging su console (default) o su un file
 * Per scegliere il file utilizzare l'opzione: 
 * -Dit.unibas.utilita.LogFile=<nomeFile>
 * es: java -Dit.unibas.utilita.Debug=finer -Dit.unibas.utilita.LogFile=e:\tmp\log.txt
 * 
 * @author G. Mecca
 * @version 1.0
 */
public class Logger {

	/**
	 * Viene utilizzato un unico logger chiamato "it.unibas.utilita"
	 */
	private static final java.util.logging.Logger logger;

	/**
	 * Il blocco di inizializzazione statica inizializza il logger, 
	 * imposta il livello di logging utilizzando la proprieta' it.unibas.utilita.Debug
	 * e sceglie il flusso su cui indirizzare il logging sulla base del valore della
	 * proprieta' it.unibas.utilita.LogFile
	 */
	static {
		logger = java.util.logging.Logger.getLogger("it.unibas.utilita");
		java.util.logging.Level livello = getLivello();
		logger.setLevel(livello);
		setHandler(logger);
	}

	/**
	 * Restituisce il livello scelto dall'utente o quello standard
	 */
	private static java.util.logging.Level getLivello() {
		String livello = System.getProperty("it.unibas.utilita.Debug");
		if (livello != null) {
			if (livello.equalsIgnoreCase("finer")) {
				return java.util.logging.Level.FINER;
			} else if (livello.equalsIgnoreCase("fine")) {
				return java.util.logging.Level.FINE;
			} else if (livello.equalsIgnoreCase("severe")) {
				return java.util.logging.Level.SEVERE;
			} else if (livello.equalsIgnoreCase("off")) {
				return java.util.logging.Level.OFF;
			} else if (livello.equalsIgnoreCase("all")) {
				return java.util.logging.Level.ALL;
			}
		}
		return java.util.logging.Level.INFO;
	}

	/**
	 * Imposta l'handler corrispondente al flusso scelto dall'utente
	 */
	private static void setHandler(java.util.logging.Logger logger){
		String nomeFile = System.getProperty("it.unibas.utilita.LogFile");
		if (nomeFile != null) {
			try {
				java.util.logging.FileHandler fileHandler = new java.util.logging.FileHandler(nomeFile);
				fileHandler.setFormatter(new java.util.logging.SimpleFormatter());
				logger.addHandler(fileHandler);
			} catch (java.io.IOException ioe) {
				logger.severe("Impossibile utilizzare il file di log specificato");
			}
		} else {
			java.util.logging.ConsoleHandler consoleHandler = new java.util.logging.ConsoleHandler();
			consoleHandler.setLevel(logger.getLevel());
			logger.addHandler(consoleHandler);
		}
	}
	
	/**
	 * Effettua il log di un messaggio con il livello "finer"
	 * @param messaggio la stringa di cui effettuare il logging
	 */
	public static void logFiner(String messaggio) {
		logger.finer(messaggio);
	}

	/**
	 * Effettua il log di un messaggio con il livello "finer" specificando
	 * inoltre la classe e il metodo da cui origina la chiamata
	 * @param classe il nome della classe da cui proviene la chiamata
	 * @param metodo il nome del metodo da cui proviene la chiamata
	 * @param messaggio la stringa di cui effettuare il logging
	 */
	public static void logFiner(String classe, String metodo, String messaggio) {
		logger.logp(java.util.logging.Level.FINER, classe, metodo, messaggio);
	}

	/**
	 * Effettua il log di un messaggio con il livello "fine"
	 * @param messaggio la stringa di cui effettuare il logging
	 */
	public static void logFine(String messaggio) {
		logger.fine(messaggio);
	}

	/**
	 * Effettua il log di un messaggio con il livello "fine" specificando
	 * inoltre la classe e il metodo da cui origina la chiamata
	 * @param classe il nome della classe da cui proviene la chiamata
	 * @param metodo il nome del metodo da cui proviene la chiamata
	 * @param messaggio la stringa di cui effettuare il logging
	 */
	public static void logFine(String classe, String metodo, String messaggio) {
		logger.logp(java.util.logging.Level.FINE, classe, metodo, messaggio);
	}

	/**
	 * Effettua il log di un messaggio con il livello "info"
	 * @param messaggio la stringa di cui effettuare il logging
	 */
	public static void logInfo(String messaggio) {
		logger.info(messaggio);
	}

	/**
	 * Effettua il log di un messaggio con il livello "info" specificando
	 * inoltre la classe e il metodo da cui origina la chiamata
	 * @param classe il nome della classe da cui proviene la chiamata
	 * @param metodo il nome del metodo da cui proviene la chiamata
	 * @param messaggio la stringa di cui effettuare il logging
	 */
	public static void logInfo(String classe, String metodo, String messaggio) {
		logger.logp(java.util.logging.Level.INFO, classe, metodo, messaggio);
	}

	/**
	 * Effettua il log di un messaggio con il livello "severe"
	 * @param messaggio la stringa di cui effettuare il logging
	 */
	public static void logSevere(String messaggio) {
		logger.severe(messaggio);
	}

	/**
	 * Effettua il log di un messaggio con il livello "severe" specificando
	 * inoltre la classe e il metodo da cui origina la chiamata
	 * @param classe il nome della classe da cui proviene la chiamata
	 * @param metodo il nome del metodo da cui proviene la chiamata
	 * @param messaggio la stringa di cui effettuare il logging
	 */
	public static void logSevere(String classe, String metodo, String messaggio) {
		logger.logp(java.util.logging.Level.SEVERE, classe, metodo, messaggio);
	}

}